#ifndef ABLAGE_H
#define ABLAGE_H
#include <condition_variable>
#include <mutex>
#include <thread>

// Mutex, um die Ausgaben von Consumer und Producer nicht zu mischen
inline std::mutex ausgabeMutex;

class Ablage {
public:
  int get()
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return belegt || beendet; });
    belegt = false;
    cond.notify_one();
    return inhalt;                           // ohne Bedeutung, falls beendet
  }

  void put(int wert)
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return !belegt || beendet; });
    belegt = true;                           // ohne Bedeutung, falls beendet
    inhalt = wert;
    cond.notify_one();
  }

  void beenden()                             // alle wait()-Aufrufe beenden
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    beendet = true;
    cond.notify_all();
  }

  bool istBeendet() const { return beendet; }

private:
  bool belegt{false};
  bool beendet{false};
  int inhalt;
  std::mutex monitorMutex;
  std::condition_variable cond;
};

#endif

