#include "Datum.h"
#include <ctime>

Datum::Datum() { aktuell(); }

void Datum::set(int t, int m, int j)
{
  if (!istGueltigesDatum(t, m, j)) {
    throw UngueltigesDatumException(t, m, j);
  }
  tag_ = t;
  monat_ = m;
  jahr_ = j;
}

void Datum::aktuell()                        // setzt #include <ctime> voraus
{
  std::time_t now{std::time(nullptr)};       // aktuelle Zeit in s seit 1.1.1970
  std::tm* z{std::localtime(&now)};          // Zeiger auf eine vordefinierte Struktur des Typs tm.
  jahr_  = z->tm_year + 1900;
  monat_ = z->tm_mon + 1;                    // localtime liefert 0..11
  tag_   = z->tm_mday;
}

Datum& Datum::operator++()                   // Tag hochzählen, präfix
{
  ++tag_;
  if (!istGueltigesDatum(tag_, monat_, jahr_)) {  // Monatsende erreicht?
    tag_ = 1;
    if (++monat_ > 12) {                     // Jahresende erreicht?
      monat_ = 1;
      ++jahr_;
    }
  }
  return *this;
}

Datum Datum::operator++(int)                 // Datum um 1 Tag erhöhen
{
  Datum temp = *this;
  ++*this;          // Präfix ++ aufrufen
  return temp;
}

Datum::operator std::string() const
{
  std::string temp("tt.mm.jjjj");
  temp[0] = tag_ / 10 + '0';                 // implizite Typumwandlung nach char
  temp[1] = tag_ % 10 + '0';
  temp[3] = monat_ / 10 + '0';
  temp[4] = monat_ % 10 + '0';
  int pos = 9;                               // letzte Jahresziffer
  int j = jahr_;
  while (j > 0) {
    temp[pos] = j % 10 + '0';                // letzte Ziffer
    j = j / 10;                              // letzte Ziffer abtrennen
    --pos;
  }
  return temp;
}


