#include <algorithm>
#include <cassert>
#include <cmath>
#include <compare>
#include <iostream>
#include <vector>

class Punkt {
public:
  constexpr Punkt(double x, double y) 
  : x_(x), y_(y) 
  { }

  constexpr double x() const { return x_;}
  constexpr double y() const { return y_;}
private:
  double x_;
  double y_;
};

constexpr auto operator<=>(const Punkt& lhs, const Punkt& rhs)
{     // Abstandsquadrate statt Abstand
  auto abstand2_lhs { lhs.x()*lhs.x() + lhs.y()*lhs.y()};
  auto abstand2_rhs { rhs.x()*rhs.x() + rhs.y()*rhs.y()};
  if (abstand2_lhs < abstand2_rhs) {
    return std::weak_ordering::less;
  } else if (abstand2_lhs > abstand2_rhs) {
    return std::weak_ordering::greater;
  }
  return std::weak_ordering::equivalent;
}

// Selbst definierter <=> erfordert zusätzlich ==
constexpr bool operator==(const Punkt& p1, const Punkt& p2) 
{
  return operator<=>(p1, p2) == 0;
}

int main()
{
  std::vector<Punkt> punkte { {1.2, 2.1}, {2.1, 1.2}, {1.0, 1.0}, 
                              {2.2, 2.2}, {2.0, 0.2}, {2.1, 1.2},
                              {0.2, 2.0}, {0.0, 0.0}, {0.1, 1.0}};
  // Test für zwei *verschiedene* Punkte mit *demselben Abstand* zum Nullpunkt:
  assert(punkte[0] == punkte[1]);  
  assert(!(punkte[0] != punkte[1]));  

  std::cout.setf(std::ios::fixed);
  std::cout.precision(1);
  for(auto p : punkte) {
    std::cout << '(' << p.x() << ", " << p.y() <<") "; 
  }
  std::cout << '\n';
  std::sort(punkte.begin(), punkte.end());
  for(auto p : punkte) {
    std::cout << '(' << p.x() << ", " << p.y() <<") "; 
  }
  std::cout << '\n';
}
